class Platform():
	registry = {}

	def __init__(self, id, name, is_brand_missing_from_name, platform_family_id, platform_family_name, model_no, storage_capacity, description, disambiguation, relevance):
		self.id = id
		self.name = name
		self.is_brand_missing_from_name = is_brand_missing_from_name
		self.platform_family_id = platform_family_id
		self.platform_family_name = platform_family_name
		self.model_no = model_no
		self.storage_capacity = storage_capacity
		self.description = description
		self.disambiguation = disambiguation
		self.relevance = relevance
		self.editions = dict()

	def add_to_registry(self):
		Platform.registry[self.id] = self

	def add_edition(self, edition):
		if edition.id not in self.editions:
			self.editions[edition.id] = edition

	def get_all_editions(self):
		return self.editions.values()

	@classmethod
	def get_by_id(cls, platform_id):
		return cls.registry[platform_id]

	@classmethod
	def get_by_edition_id(cls, edition_id):
		for platform in cls.get_all():
			for edition in platform.editions:
				if edition == edition_id:
					return platform
		raise Exception()

	@classmethod
	def get_all(cls):
		return cls.registry.values()



class Board():
	registry = {}

	def __init__(self, id, name, url, is_scraping_supported):
		self.id = id
		self.name = name
		self.url = url
		self.is_scraping_supported = is_scraping_supported
		self.listings = dict()
		self.add_to_registry()

	def add_to_registry(self):
		Board.registry[self.id] = self
		
	def add_listing(self, listing):
		if listing.id not in self.listings:
			self.listings[listing.id] = listing

	def get_all_listings(self):
		return self.listings.values()


	def summary(self):
		print('id: ' + str(self.id))
		print('name: ' + self.name)
		print('url: ' + self.url)
		print('is_scraping_supported: ' + str(self.is_scraping_supported))

	@classmethod
	def get_by_id(cls, board_id):
		return cls.registry[board_id]

	@classmethod
	def get_all(cls):
		return cls.registry.values()



class PlatformNameGroup():
	registry = {}

	def __init__(self, id, name, description):
		self.id = id
		self.name = name
		self.description = description
		self.platforms = dict()

	def add_to_registry(self):
		PlatformNameGroup.registry[self.id] = self
		
	def add_platform(self, platform):
		if platform.id not in self.platforms:
			self.platforms[platform.id] = platform

	def get_all_platforms(self):
		return self.platforms.values()


	@classmethod
	def get_by_id(cls, platform_name_group_id):
		return cls.registry[platform_name_group_id]

	@classmethod
	def get_all(cls):
		return cls.registry.values()



class Game():
	registry = {}

	def __init__(self, id, name, year_first_release, is_bootleg):
		self.id = id
		self.name = name
		self.year_first_release = year_first_release
		self.is_bootleg = is_bootleg

	def add_to_registry(self):
		Game.registry[self.id] = self

	@classmethod
	def get_by_id(cls, game_id):
		return cls.registry[game_id]

	@classmethod
	def get_all(cls):
		return cls.registry.values()



class PlatformEdition():
	registry = {}

	def __init__(self, id, name, official_color, has_matte, has_transparency, has_gloss, note, image_url):
		self.id = id
		self.name = name
		self.official_color = official_color
		self.has_matte = has_matte
		self.has_transparency = has_transparency
		self.has_gloss = has_gloss
		self.note = note
		self.image_url = image_url
		self.colors = list()

	def add_to_registry(self):
		PlatformEdition.registry[self.id] = self

	@classmethod
	def get_by_id(cls, platform_edition_id):
		return cls.registry[platform_edition_id]

	@classmethod
	def get_all(cls):
		return cls.registry.values()



class User():
	registry = {}

	def __init__(self, id, public_id, username, email, password, created_at):
		self.id = id
		self.public_id = public_id
		self.username = username
		self.email = email
		self.password = password
		self.created_at = created_at

	def add_to_registry(self):
		User.registry[self.id] = self

	@classmethod
	def get_by_id(cls, user_id):
		return cls.registry[user_id]

	@classmethod
	def get_all(cls):
		return cls.registry.values()



class Listing():
	registry = {}

	def __init__(self, id, native_id, title, body, price, url, date_posted, date_scraped):
		self.id = id
		self.native_id = native_id
		self.title = title
		self.body = body
		self.price = price
		self.url = url
		self.date_posted = date_posted
		self.date_scraped = date_scraped

	def add_to_registry(self):
		Listing.registry[self.id] = self

	@classmethod
	def get_by_id(cls, listing_id):
		return cls.registry[listing_id]

	@classmethod
	def get_all(cls):
		return cls.registry.values()
